@@ -3,6 +3,7 @@
 #include <string.h>
 #include <assert.h>
 #include <dirent.h>
+#include <libgen.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <sys/types.h>
@@ -64,6 +65,19 @@ static int extend_file(struct thread_data *td, struct fio_file *f)
 		return 1;
 	}
 
+#ifdef FIO_HAVE_FALLOCATE
+	if (td->o.fallocate && !td->o.fill_device) {
+		dprint(FD_FILE, "fallocate file %s size %llu\n", f->file_name,
+							f->real_file_size);
+
+		r = posix_fallocate(f->fd, 0, f->real_file_size);
+		if (r < 0) {
+			log_err("fio: posix_fallocate fails: %s\n",
+					strerror(-r));
+		}
+	}
+#endif
+	
 	if (!new_layout)
 		goto done;
 
@@ -78,16 +92,6 @@ static int extend_file(struct thread_data *td, struct fio_file *f)
 			td_verror(td, errno, "ftruncate");
 			goto err;
 		}
-
-#ifdef FIO_HAVE_FALLOCATE
-		dprint(FD_FILE, "fallocate file %s, size %llu\n", f->file_name,
-							f->real_file_size);
-		r = posix_fallocate(f->fd, 0, f->real_file_size);
-		if (r < 0) {
-			log_err("fio: posix_fallocate fails: %s\n",
-					strerror(-r));
-		}
-#endif
 	}
 
 	b = malloc(td->o.max_bs[DDIR_WRITE]);
@@ -500,6 +504,75 @@ static int get_file_sizes(struct thread_data *td)
 	return err;
 }
 
+struct fio_mount {
+	struct flist_head list;
+	const char *base;
+	char __base[256];
+	unsigned int key;
+};
+
+/*
+ * Get free number of bytes for each file on each unique mount.
+ */
+static unsigned long long get_fs_free_counts(struct thread_data *td)
+{
+	struct flist_head *n, *tmp;
+	unsigned long long ret;
+	struct fio_mount *fm;
+	FLIST_HEAD(list);
+	struct fio_file *f;
+	unsigned int i;
+
+	for_each_file(td, f, i) {
+		struct stat sb;
+		char buf[256];
+
+		strcpy(buf, f->file_name);
+
+		if (stat(buf, &sb) < 0) {
+			if (errno != ENOENT)
+				break;
+			strcpy(buf, ".");
+			if (stat(buf, &sb) < 0)
+				break;
+		}
+
+		fm = NULL;
+		flist_for_each(n, &list) {
+			fm = flist_entry(n, struct fio_mount, list);
+			if (fm->key == sb.st_dev)
+				break;
+
+			fm = NULL;
+		}
+
+		if (fm)
+			continue;
+
+		fm = malloc(sizeof(*fm));
+		strcpy(fm->__base, buf);
+		fm->base = basename(fm->__base);
+		fm->key = sb.st_dev;
+		flist_add(&fm->list, &list);
+	}
+
+	ret = 0;
+	flist_for_each_safe(n, tmp, &list) {
+		unsigned long long sz;
+
+		fm = flist_entry(n, struct fio_mount, list);
+		flist_del(&fm->list);
+
+		sz = get_fs_size(fm->base);
+		if (sz && sz != -1ULL)
+			ret += sz;
+
+		free(fm);
+	}
+
+	return ret;
+}
+
 /*
  * Open the files and setup files sizes, creating files if necessary.
  */
@@ -540,6 +613,9 @@ int setup_files(struct thread_data *td)
 			total_size += f->real_file_size;
 	}
 
+	if (td->o.fill_device)
+		td->fill_device_size = get_fs_free_counts(td);
+
 	/*
 	 * device/file sizes are zero and no size given, punt
 	 */
@@ -743,11 +819,8 @@ void close_and_free_files(struct thread_data *td)
 
 		sfree(f->file_name);
 		f->file_name = NULL;
-
-		if (f->file_map) {
-			sfree(f->file_map);
-			f->file_map = NULL;
-		}
+		sfree(f->file_map);
+		f->file_map = NULL;
 		sfree(f);
 	}
 
@@ -767,7 +840,7 @@ static void get_file_type(struct fio_file *f)
 	else
 		f->filetype = FIO_TYPE_FILE;
 
-	if (!lstat(f->file_name, &sb)) {
+	if (!stat(f->file_name, &sb)) {
 		if (S_ISBLK(sb.st_mode))
 			f->filetype = FIO_TYPE_BD;
 		else if (S_ISCHR(sb.st_mode))
