@@ -28,8 +28,10 @@
 #include "io_ddir.h"
 #include "ioengine.h"
 #include "iolog.h"
-#include "profiles.h"
 #include "helpers.h"
+#include "options.h"
+#include "profile.h"
+#include "time.h"
 
 #ifdef FIO_HAVE_GUASI
 #include <guasi.h>
@@ -183,6 +185,8 @@ struct thread_options {
 	unsigned int verify_pattern_bytes;
 	unsigned int verify_fatal;
 	unsigned int verify_async;
+	unsigned long long verify_backlog;
+	unsigned int verify_batch;
 	unsigned int use_thread;
 	unsigned int unlink;
 	unsigned int do_disk_util;
@@ -237,6 +241,7 @@ struct thread_options {
 	unsigned int gtod_reduce;
 	unsigned int gtod_cpu;
 	unsigned int gtod_offload;
+	enum fio_cs clocksource;
 
 	char *read_iolog_file;
 	char *write_iolog_file;
@@ -271,16 +276,19 @@ struct thread_options {
 	/*
 	 * Benchmark profile type
 	 */
-	unsigned int profile;
+	char *profile;
 
 	/*
 	 * blkio cgroup support
 	 */
 	char *cgroup;
 	unsigned int cgroup_weight;
+	unsigned int cgroup_nodelete;
 
 	unsigned int uid;
 	unsigned int gid;
+
+	unsigned int sync_file_range;
 };
 
 #define FIO_VERROR_SIZE	128
@@ -315,6 +323,7 @@ struct thread_data {
 	unsigned int ioprio;
 	unsigned int ioprio_set;
 	unsigned int last_was_sync;
+	enum fio_ddir last_ddir;
 
 	char *mmapfile;
 	int mmapfd;
@@ -329,6 +338,8 @@ struct thread_data {
 	os_random_state_t bsrange_state;
 	os_random_state_t verify_state;
 
+	unsigned int verify_batch;
+
 	int shm_id;
 
 	/*
@@ -367,6 +378,7 @@ struct thread_data {
 	struct timeval lastrate[2];
 
 	unsigned long long total_io_size;
+	unsigned long long fill_device_size;
 
 	unsigned long io_issues[2];
 	unsigned long long io_blocks[2];
@@ -403,6 +415,7 @@ struct thread_data {
 	 */
 	struct rb_root io_hist_tree;
 	struct flist_head io_hist_list;
+	unsigned long io_hist_len;
 
 	/*
 	 * For IO replaying
@@ -416,6 +429,8 @@ struct thread_data {
 	unsigned int file_service_left;
 	struct fio_file *file_service_file;
 
+	unsigned int sync_file_range_nr;
+
 	/*
 	 * For generating file sizes
 	 */
@@ -426,6 +441,12 @@ struct thread_data {
 	 */
 	unsigned int total_err_count;
 	int first_error;
+
+	/*
+	 * Can be overloaded by profiles
+	 */
+	struct prof_io_ops prof_io_ops;
+	void *prof_data;
 };
 
 /*
@@ -470,6 +491,7 @@ extern unsigned long done_secs;
 extern char *job_section;
 extern int fio_gtod_offload;
 extern int fio_gtod_cpu;
+extern enum fio_cs fio_clock_source;
 
 extern struct thread_data *threads;
 
@@ -506,25 +528,6 @@ static inline int should_fsync(struct thread_data *td)
 	return 0;
 }
 
-/*
- * Time functions
- */
-extern unsigned long long utime_since(struct timeval *, struct timeval *);
-extern unsigned long long utime_since_now(struct timeval *);
-extern unsigned long mtime_since(struct timeval *, struct timeval *);
-extern unsigned long mtime_since_now(struct timeval *);
-extern unsigned long time_since_now(struct timeval *);
-extern unsigned long mtime_since_genesis(void);
-extern void usec_spin(unsigned int);
-extern void usec_sleep(struct thread_data *, unsigned long);
-extern void fill_start_time(struct timeval *);
-extern void fio_gettime(struct timeval *, void *);
-extern void fio_gtod_init(void);
-extern void fio_gtod_update(void);
-extern void set_genesis_time(void);
-extern int ramp_time_over(struct thread_data *);
-extern int in_ramp_time(struct thread_data *);
-
 /*
  * Init/option functions
  */
@@ -538,8 +541,9 @@ extern void fio_options_dup_and_init(struct option *);
 extern void options_mem_dupe(struct thread_data *);
 extern void options_mem_free(struct thread_data *);
 extern void td_fill_rand_seeds(struct thread_data *);
+extern void add_job_opts(const char **);
 #define FIO_GETOPT_JOB		0x89988998
-#define FIO_NR_OPTIONS		128
+#define FIO_NR_OPTIONS		(FIO_MAX_OPTS + 128)
 
 /*
  * ETA/status stuff
